# predictram-parameter-data/query_parser.py
import re

# Mapping of display names to JSON keys
METRIC_MAPPING = {
    # Stock / Financial Metrics
    "Stock Industry": "industry",
    "Dividend Payout Ratio": "payoutRatio",
    "Five Year Avg Dividend Yield": "fiveYearAvgDividendYield",
    "Price to Sales Trailing 12 Months": "priceToSalesTrailing12Months",
    "Trailing Annual Dividend Rate": "trailingAnnualDividendRate",
    "Profit Margins": "profitMargins",
    "% Held by Insiders": "heldPercentInsiders",
    "% Held by Institutions": "heldPercentInstitutions",
    "Volume": "volume",
    "Regular Market Volume": "regularMarketVolume",
    "Average Volume": "averageVolume",
    "Average Volume 10 days": "averageVolume10days",
    "Average Daily Volume 10 Day": "averageDailyVolume10Day",
    "Return on Assets": "returnOnAssets",
    "Return on Assets (ttm)": "returnOnAssets",
    "Return on Equity (ttm)": "returnOnEquity",
    "Total Value": "totalValue",
    "Correlation with NSEI": "Correlation with ^NSEI",
    "Annualized Alpha (%)": "Annualized Alpha (%)",
    "Annualized Volatility (%)": "Annualized Volatility (%)",
    "Sharpe Ratio": "Sharpe Ratio",
    "Treynor Ratio": "Treynor Ratio",
    "Sortino Ratio": "Sortino Ratio",
    "Maximum Drawdown": "Maximum Drawdown",
    "R-Squared": "R-Squared",
    "Downside Deviation": "Downside Deviation",
    "Annualized Tracking Error (%)": "Annualized Tracking Error (%)",
    "VaR (95%)": "VaR (95%)",
    "50-Day Moving Average": "50-Day Moving Average",
    "200-Day Moving Average": "200-Day Moving Average",
    "New Haircut Margin": "New Haircut Margin",
    "Rating": "Rating",
    "Combined Score": "Combined Score",
    "New Collateral Value Percentage": "New Collateral Value Percentage",
    "Market Cap": "marketCap",
    "Enterprise Value": "enterpriseValue",
    "Trailing P/E": "trailingPE",
    "Forward P/E": "forwardPE",
    "PEG Ratio": "pegRatio",
    "Price/Sales": "priceToSalesTrailing12Months",
    "Price/Book": "priceToBook",
    "Enterprise Value/Revenue": "enterpriseToRevenue",
    "Enterprise Value/EBITDA": "enterpriseToEbitda",
    "Beta": "beta",
    "52-Week High": "fiftyTwoWeekHigh",
    "52-Week Low": "fiftyTwoWeekLow",
    "50-Day Average": "fiftyDayAverage",
    "200-Day Average": "twoHundredDayAverage",
    "Short Ratio": "shortRatio",
    "Short % of Float": "shortPercentOfFloat",
    "Shares Outstanding": "sharesOutstanding",
    "Float": "floatShares",
    "Shares Short": "sharesShort",
    "Short Ratio": "shortRatio",
    "Book Value": "bookValue",
    "Price/Book": "priceToBook",
    "EBITDA": "ebitda",
    "Revenue": "revenue",
    "Revenue Per Share": "revenuePerShare",
    "Gross Profit": "grossProfit",
    "Free Cash Flow": "freeCashflow",
    "Operating Cash Flow": "operatingCashflow",
    "Earnings Growth": "earningsGrowth",
    "Revenue Growth": "revenueGrowth",
    "Current Ratio": "currentRatio",
    "Quick Ratio": "quickRatio",
    "Debt to Equity": "debtToEquity",
    "Total Debt": "totalDebt",
    "Total Cash": "totalCash",
    "Total Cash Per Share": "totalCashPerShare",
    "CAGR": "CAGR",
    "ROI": "ROI",
    "EPS": "trailingEps",
    "EPS Growth": "epsGrowth",

    # Interpretations
    "Correlation with NSEI Interpretation": "Correlation with ^NSEI Interpretation",
    "Alpha Interpretation": "Annualized Alpha (%) Interpretation",
    "Volatility Interpretation": "Annualized Volatility (%) Interpretation",
    "Sharpe Ratio Interpretation": "Sharpe Ratio Interpretation",
    "Treynor Ratio Interpretation": "Treynor Ratio Interpretation",
    "Sortino Ratio Interpretation": "Sortino Ratio Interpretation",
    "Maximum Drawdown Interpretation": "Maximum Drawdown Interpretation",
    "R-Squared Interpretation": "R-Squared Interpretation",
    "Downside Deviation Interpretation": "Downside Deviation Interpretation",
    "Tracking Error Interpretation": "Annualized Tracking Error (%) Interpretation",
    "VaR Interpretation": "VaR (95%) Interpretation",
    "Moving Average Interpretation": "Moving Average Interpretation",
    "Valuation Interpretation": "Valuation Interpretation",

    # Company Info
    "Address": "address1",
    "City": "city",
    "Zip": "zip",
    "Country": "country",
    "Website": "website",
    "Sector": "sector",
    "Industry": "industry",
    "Business Summary": "longBusinessSummary",
    "Full Time Employees": "fullTimeEmployees",
    "Company Name": "shortName",
    "Exchange": "exchange",
    "Currency": "currency",
    "Quote Type": "quoteType",

    # Financial Statements
    "Income Statement (Quarterly)": "incomeStatementQuarterly",
    "Income Statement (Annual)": "incomeStatementAnnual",
    "Balance Sheet (Quarterly)": "balanceSheetQuarterly",
    "Balance Sheet (Annual)": "balanceSheetAnnual",
    "Cash Flow (Quarterly)": "cashflowStatementQuarterly",
    "Cash Flow (Annual)": "cashflowStatementAnnual",

    # Industry Comparisons
    "Industry Forward PE": "industry_forwardPE",
    "Industry Trailing PE": "industry_trailingPE",
    "Industry Debt to Equity": "industry_debtToEquity",
    "Industry Current Ratio": "industry_currentRatio",
    "Industry Quick Ratio": "industry_quickRatio",
    "Industry EBITDA": "industry_ebitda",
    "Industry Total Debt": "industry_totalDebt",
    "Industry Return on Assets": "industry_returnOnAssets",
    "Industry Return on Equity": "industry_returnOnEquity",
    "Industry Revenue Growth": "industry_revenueGrowth",
    "Industry Gross Margins": "industry_grossMargins",
    "Industry EBITDA Margins": "industry_ebitdaMargins",
    "Industry Operating Margins": "industry_operatingMargins",
    "Industry PEG Ratio": "industry_pegRatio",
    "Industry Price/Sales": "industry_priceToSales",
    "Industry Price/Book": "industry_priceToBook",

    # Alternative Names
    "PE Ratio": "trailingPE",
    "Price to Earnings": "trailingPE",
    "Price to Book Ratio": "priceToBook",
    "Price to Sales Ratio": "priceToSalesTrailing12Months",
    "Debt/Equity": "debtToEquity",
    "Current Assets": "totalCurrentAssets",
    "Current Liabilities": "totalCurrentLiabilities",
    "Total Assets": "totalAssets",
    "Total Liabilities": "totalLiabilities",
    "Shareholders Equity": "totalStockholderEquity",
    "Operating Income": "operatingIncome",
    "Net Income": "netIncome",
    "Diluted EPS": "trailingEps",
    "Basic EPS": "trailingEps",
    "Dividend Yield": "dividendYield",
    "Payout Ratio": "payoutRatio",
    "Enterprise Value to EBITDA": "enterpriseToEbitda",
    "Enterprise Value to Revenue": "enterpriseToRevenue",
    "EV/EBITDA": "enterpriseToEbitda",
    "EV/Revenue": "enterpriseToRevenue",
    "Quick Ratio": "quickRatio",
    "Acid Test Ratio": "quickRatio",
    "Working Capital": "workingCapital"
}

def parse_query(query_string):
    """
    Parse a query string into a list of conditions
    
    Args:
        query_string (str): The query string to parse
        
    Returns:
        list: List of conditions (dicts with metric, operator, value)
    """
    # Split into individual conditions
    conditions = re.split(r'\s+(?:and|or)\s+', query_string, flags=re.IGNORECASE)
    
    parsed_conditions = []
    
    for condition in conditions:
        # Match metric operator value pattern
        match = re.match(
            r'^(.*?)\s*(>=|<=|>|<|=|!=|contains|like)\s*(.*?)$', 
            condition.strip(), 
            flags=re.IGNORECASE
        )
        
        if not match:
            raise ValueError(f"Could not parse condition: {condition}")
            
        metric, operator, value = match.groups()
        
        # Clean up metric name
        metric = metric.strip()
        
        # Get the JSON key for this metric
        json_key = METRIC_MAPPING.get(metric, metric)
        
        # Clean and convert value
        try:
            # Remove % signs and convert to float if numeric
            if '%' in value:
                value = float(value.replace('%', '')) / 100
            else:
                value = float(value)
        except ValueError:
            # If not numeric, keep as string
            value = value.strip('"\'')
        
        parsed_conditions.append({
            'metric': metric,
            'json_key': json_key,
            'operator': operator.lower(),
            'value': value
        })
    
    return parsed_conditions